      SUBROUTINE SOB_COMP ( NX, XI, ATT, TEM, OPA, TAT, IUER )
! ************************************************************************
! *                                                                      *
! *   Routine SOB_COMP computes atmosphere opacity (i.e. optical depth)  *
! *   and atmosphere brightness temperature along the trajectory XI.     *
! *                                                                      *
! *   NB: For reasons which are not understood, SOB_COMP should be       *
! *       compiled with optimizations turned off.                        *
! *                                                                      *
! * _________________________ Input parameters: ________________________ *
! *                                                                      *
! *   NH ( INTEGER*4  ) -- The number of knots of the B-spline expansion *
! *                        of refractivity along XI axis.                *
! *   XI ( REAL*8     ) -- Array of knots along the XI axis. The first   *
! *                        point corresponds to the receiver. The last   *
! *                        point corresponds to the top of the           *
! *                        atmosphere in the direction to the emitter,   *
! *                        undisturbed by refraction. Dimension: NH.     *
! *                        Units: m.                                     *
! *  ATT ( REAL*8     ) -- Array of atmosphere attenuation (absorption)  *
! *                        at a specific frequency. Dimension: NH.       *
! *                        Units: 1/m. Att=1 means the electromagnetic   *
! *                        wave is attenuated by the factor of 2.7183    *
! *                        for propagation through 1 meter of the media. *
! *  TEM ( REAL*8     ) -- Array of atmosphere air temperature.          *
! *                        Dimension: NH. Units: K.                      *
! *                                                                      *
! * _________________________ Output parameters: _______________________ *
! *                                                                      *
! *  OPA ( REAL*8     ) -- Array of atmosphere opacity for the distance  *
! *                        XI(1) to XI(NH) /from the ground station to   *
! *                        the top of the atmosphere/. Opacity 1 means   *
! *                        the ratio of power at the top of the          *
! *                        atmosphere to the power at the ground is      *
! *                        2.7183. Units: dimensionless.                 *
! *  TAT ( REAL*8     ) -- Brightness temperature of the atmosphere      *
! *                        at the ground. Unit: K.                       *
! *                                                                      *
! * ________________________ Modified parameters: ______________________ *
! *                                                                      *
! *     IUER ( INTEGER*4, OPT ) -- Universal error handler.              *
! *                           Input: switch IUER=0 -- no error messages  *
! *                                  will be generated even in the case  *
! *                                  of error. IUER=-1 -- in the case of *
! *                                  error the message will be put on    *
! *                                  stdout.                             *
! *                           Output: 0 in the case of successful        *
! *                                   completion and non-zero in the     *
! *                                   case of error.                     *
! *                                                                      *
! *  ### 12-SEP-2014    SOB_COMP   v1.0 (c)  L. Petrov  17-SEP-2014 ###  *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'spd.i'
      INTEGER*4  NX, IUER
      REAL*8     XI(NX), ATT(NX), TEM(NX), OPA, TAT 
      REAL*8     ATT_SPL(1-SPD__MDEG:SPD__MLEV), &
     &           FUN_SPL(1-SPD__MDEG:SPD__MLEV), OPA_FUN(SPD__MLEV)
      REAL*8     FUN_SPL_SAVE(1-SPD__MDEG:SPD__MLEV)
      REAL*8     OPA_MIN
      PARAMETER  ( OPA_MIN = 1.D-12 )
      INTEGER*4  J1, J2, J3, IND, INDF, IER
      LOGICAL*4, EXTERNAL :: IS_R8_NAN 
      REAL*8,    EXTERNAL :: VAL_BSPLJ 
!
! --- Fill array ATT_SPL and FUN_SPL
!
      DO 410 J1=1,NX
         ATT_SPL(J1) = ATT(J1)
 410  CONTINUE 
!
! --- Expand attenuation into B-spline basus
!
      CALL ERR_PASS ( IUER, IER )
      CALL BSPL_1D_CMP ( SPD__MDEG, 0, NX, XI, ATT_SPL, IER )
      IF ( IER .NE. 0 ) THEN
           CALL ERR_LOG ( 5591, IUER, 'SOB_COMP', 'Error in '// &
     &         'an attempt to compute expansion of the atmosphere '// &
     &         'attenuation into B-spline basis' )
           RETURN 
      END IF
!
! --- Computee opacity at the ground by ingegratin of attenuation
! --- along axis XI
!
      IND = 1
      OPA  = VAL_BSPLJ ( XI(IND), SPD__MDEG, NX, IND, XI, ATT_SPL )
      INDF = 0
!
! --- Compute array of OPA_FUN (optical depth) and 
! --- FUN_SPL -- product of air termperature and accumulated attenuation
!
! --- NB: integration of radiative transfer equation is performed from
! --- the top of the atmosphere down to the station. Therefore, we 
! --- re-index arrays
!
      OPA_FUN(IND) = 0.0
      FUN_SPL(IND) = TEM(NX)*DEXP(-OPA_FUN(IND))
      DO 420 J2=NX-1,1,-1 ! NB: cycle runs backward
!
! ------ Compute optical depth between node J2 and the top of the atmosphere
!
         IND = IND + 1
         OPA_FUN(IND) = VAL_BSPLJ ( XI(J2), SPD__MDEG, NX, J2, XI, ATT_SPL )
!
         IF ( IS_R8_NAN ( OPA_FUN(IND) ) ) THEN
              WRITE ( 6, * ) 'IND = ', IND, ' J2= ', J2, ' NX= ', NX
              WRITE ( 6, * ) 'XI      = ', XI
              WRITE ( 6, * ) 'ATT     = ', ATT
              WRITE ( 6, * ) 'ATT_SPL = ', ATT_SPL
              WRITE ( 6, * ) 'OPA_FUN = ', OPA_FUN
              CALL EXIT ( 1 )
         END IF
!
! ------ Check whether opacity is small
! ------ This is done to avoid out-of-order sequence due to rounding errors
!
         IF ( OPA_FUN(IND) > SPD__OPA_MIN ) THEN
              IF ( INDF == 0 ) INDF = IND
         END IF
!
! ------ Compute the function under integral
!
         FUN_SPL(IND) = TEM(J2)*DEXP(-OPA_FUN(IND))
 420  CONTINUE 
 820  CONTINUE 
!
! --- Sort arrays OPA_FUN/FUN_SPL. Due to roundig errors array OPA_FUN
! --- may not be an increasing sequency. Check and correct this.
!
      CALL SORT8 ( NX-INDF+1, OPA_FUN(INDF), FUN_SPL(INDF) )
      DO 430 J3=INDF+1,NX-INDF
!
! ------ Difference in OPA_FUN adgument should be no less than OPA_MIN
!
         IF ( OPA_FUN(J3) - OPA_FUN(J3-1) < OPA_MIN )  THEN
              OPA_FUN(J3) = OPA_FUN(J3-1) + OPA_MIN
         END IF
 430  CONTINUE 
      FUN_SPL_SAVE = FUN_SPL ! %%%%%%%%%%%
!
! --- We compute the B-spline expansion for a section of arrays OPA_FUN, FUN_SPL
! --- starting from index IND
!
      CALL ERR_PASS ( IUER, IER )
      CALL BSPL_1D_CMP ( SPD__MDEG, 0, NX-INDF+1, OPA_FUN(INDF), FUN_SPL(INDF), &
     &                   IER )
      IF ( IER .NE. 0 ) THEN
           write ( 6, * ) ' nx = ', nx, ' indf= ', indf
           do 520 j2=1,nx
              write ( 6, 210 ) j2, xi(j2), att(j2), tem(j2), opa_fun(j2), fun_spl_save(j2)
 210          format ( 'i= ',i3, ' xi/tem/opa/fun= ', 5(1pd22.15,1x) )
 520       continue 
           write ( 6, * ) ' ier= ', ier, ' iuer= ', iuer 
           CALL ERR_LOG ( 5593, IUER, 'SOB_COMP', 'Error in '// &
     &         'an attempt to compute expansion of the function that '// &
     &         'is the product of accumulated atmosphere opacity and '// &
     &         'thermodynamic temperature into B-spline basis' )
           RETURN 
      END IF
!
! --- Finally, get the atmosphere brightness temperature at the station
!
      TAT = VAL_BSPLJ ( OPA_FUN(INDF), SPD__MDEG, NX-INDF+1, 1, OPA_FUN(INDF), FUN_SPL(INDF) )
!
      CALL ERR_LOG ( 0, IUER )
      RETURN
      END  SUBROUTINE  SOB_COMP  !#!  
